# -*- coding: utf-8 -*-
#  Copyright (c) 2021. Distributed under the terms of the MIT License.

from scipy.constants import physical_constants as pc
au_to_angstrom = pc["atomic unit of length"][0] * 10 ** 10  # = 0.5291772109
