# -*- coding: utf-8 -*-
#  Copyright (c) 2021. Distributed under the terms of the MIT License.
from typing import Tuple, List, Union

Coords = Tuple[float, float, float]
GenCoords = Union[Tuple[float, float, float], List[float]]
